@@ -1,10 +1,10 @@
 """Xonsh color styling tools that simulate pygments, when it is unavailable."""
-import builtins
+
 from collections import defaultdict
 
+from xonsh.color_tools import RE_BACKGROUND, iscolor, warn_deprecated_no_color
+from xonsh.lib.lazyasd import LazyObject
 from xonsh.platform import HAS_PYGMENTS
-from xonsh.lazyasd import LazyObject
-from xonsh.color_tools import RE_BACKGROUND
 from xonsh.tools import FORMATTER
 
 
@@ -16,7 +16,7 @@ class _TokenType(tuple):
     See https://bitbucket.org/birkenfeld/pygments-main/raw/05818a4ef9891d9ac22c851f7b3ea4b4fce460ab/AUTHORS
     """
 
-    parent = None
+    parent: "_TokenType|None" = None
 
     def split(self):
         buf = []
@@ -64,17 +64,19 @@ def partial_color_tokenize(template):
     of tuples mapping the token to the string which has that color.
     These sub-strings maybe templates themselves.
     """
-    if HAS_PYGMENTS and builtins.__xonsh__.shell is not None:
-        styles = __xonsh__.shell.shell.styler.styles
-    elif builtins.__xonsh__.shell is not None:
+    from xonsh.built_ins import XSH
+
+    if HAS_PYGMENTS and XSH.shell is not None:
+        styles = XSH.shell.shell.styler.styles
+    elif XSH.shell is not None:
         styles = DEFAULT_STYLE_DICT
     else:
         styles = None
-    color = Color.NO_COLOR
+    color = Color.RESET
     try:
         toks, color = _partial_color_tokenize_main(template, styles)
     except Exception:
-        toks = [(Color.NO_COLOR, template)]
+        toks = [(Color.RESET, template)]
     if styles is not None:
         styles[color]  # ensure color is available
     return toks
@@ -85,14 +87,14 @@ def _partial_color_tokenize_main(template, styles):
     bclose = "}"
     colon = ":"
     expl = "!"
-    color = Color.NO_COLOR
+    color = Color.RESET
     fg = bg = None
     value = ""
     toks = []
     for literal, field, spec, conv in FORMATTER.parse(template):
         if field is None:
             value += literal
-        elif field in KNOWN_COLORS or "#" in field:
+        elif iscolor(field):
             value += literal
             next_color, fg, bg = color_by_name(field, fg, bg)
             if next_color is not color:
@@ -142,8 +144,10 @@ def color_by_name(name, fg=None, bg=None):
         New computed background color name.
     """
     name = name.upper()
-    if name == "NO_COLOR":
-        return Color.NO_COLOR, None, None
+    if name in ("RESET", "NO_COLOR"):
+        if name == "NO_COLOR":
+            warn_deprecated_no_color()
+        return Color.RESET, None, None
     m = RE_BACKGROUND.search(name)
     if m is None:  # must be foreground color
         fg = norm_name(name)
@@ -151,7 +155,7 @@ def color_by_name(name, fg=None, bg=None):
         bg = norm_name(name)
     # assemble token
     if fg is None and bg is None:
-        tokname = "NO_COLOR"
+        tokname = "RESET"
     elif fg is None:
         tokname = bg
     elif bg is None:
@@ -164,106 +168,21 @@ def color_by_name(name, fg=None, bg=None):
 
 def norm_name(name):
     """Normalizes a color name."""
-    return name.replace("#", "HEX").replace("BGHEX", "BACKGROUND_HEX")
+    return name.upper().replace("#", "HEX")
 
 
-KNOWN_COLORS = LazyObject(
-    lambda: frozenset(
-        [
-            "BACKGROUND_BLACK",
-            "BACKGROUND_BLUE",
-            "BACKGROUND_CYAN",
-            "BACKGROUND_GREEN",
-            "BACKGROUND_INTENSE_BLACK",
-            "BACKGROUND_INTENSE_BLUE",
-            "BACKGROUND_INTENSE_CYAN",
-            "BACKGROUND_INTENSE_GREEN",
-            "BACKGROUND_INTENSE_PURPLE",
-            "BACKGROUND_INTENSE_RED",
-            "BACKGROUND_INTENSE_WHITE",
-            "BACKGROUND_INTENSE_YELLOW",
-            "BACKGROUND_PURPLE",
-            "BACKGROUND_RED",
-            "BACKGROUND_WHITE",
-            "BACKGROUND_YELLOW",
-            "BLACK",
-            "BLUE",
-            "BOLD_BLACK",
-            "BOLD_BLUE",
-            "BOLD_CYAN",
-            "BOLD_GREEN",
-            "BOLD_INTENSE_BLACK",
-            "BOLD_INTENSE_BLUE",
-            "BOLD_INTENSE_CYAN",
-            "BOLD_INTENSE_GREEN",
-            "BOLD_INTENSE_PURPLE",
-            "BOLD_INTENSE_RED",
-            "BOLD_INTENSE_WHITE",
-            "BOLD_INTENSE_YELLOW",
-            "BOLD_PURPLE",
-            "BOLD_RED",
-            "BOLD_UNDERLINE_BLACK",
-            "BOLD_UNDERLINE_BLUE",
-            "BOLD_UNDERLINE_CYAN",
-            "BOLD_UNDERLINE_GREEN",
-            "BOLD_UNDERLINE_INTENSE_BLACK",
-            "BOLD_UNDERLINE_INTENSE_BLUE",
-            "BOLD_UNDERLINE_INTENSE_CYAN",
-            "BOLD_UNDERLINE_INTENSE_GREEN",
-            "BOLD_UNDERLINE_INTENSE_PURPLE",
-            "BOLD_UNDERLINE_INTENSE_RED",
-            "BOLD_UNDERLINE_INTENSE_WHITE",
-            "BOLD_UNDERLINE_INTENSE_YELLOW",
-            "BOLD_UNDERLINE_PURPLE",
-            "BOLD_UNDERLINE_RED",
-            "BOLD_UNDERLINE_WHITE",
-            "BOLD_UNDERLINE_YELLOW",
-            "BOLD_WHITE",
-            "BOLD_YELLOW",
-            "CYAN",
-            "GREEN",
-            "INTENSE_BLACK",
-            "INTENSE_BLUE",
-            "INTENSE_CYAN",
-            "INTENSE_GREEN",
-            "INTENSE_PURPLE",
-            "INTENSE_RED",
-            "INTENSE_WHITE",
-            "INTENSE_YELLOW",
-            "NO_COLOR",
-            "PURPLE",
-            "RED",
-            "UNDERLINE_BLACK",
-            "UNDERLINE_BLUE",
-            "UNDERLINE_CYAN",
-            "UNDERLINE_GREEN",
-            "UNDERLINE_INTENSE_BLACK",
-            "UNDERLINE_INTENSE_BLUE",
-            "UNDERLINE_INTENSE_CYAN",
-            "UNDERLINE_INTENSE_GREEN",
-            "UNDERLINE_INTENSE_PURPLE",
-            "UNDERLINE_INTENSE_RED",
-            "UNDERLINE_INTENSE_WHITE",
-            "UNDERLINE_INTENSE_YELLOW",
-            "UNDERLINE_PURPLE",
-            "UNDERLINE_RED",
-            "UNDERLINE_WHITE",
-            "UNDERLINE_YELLOW",
-            "WHITE",
-            "YELLOW",
-        ]
-    ),
-    globals(),
-    "KNOWN_COLORS",
-)
+def style_as_faded(template: str) -> str:
+    """Remove the colors from the template string and style as faded."""
+    tokens = partial_color_tokenize(template)
+    without_color = "".join([str(sect) for _, sect in tokens])
+    return "{RESET}{#d3d3d3}" + without_color + "{RESET}"
+
 
 DEFAULT_STYLE_DICT = LazyObject(
     lambda: defaultdict(
         lambda: "",
         {
             Token: "",
-            Token.Aborted: "ansibrightblack",
-            Token.AutoSuggestion: "ansibrightblack",
             Token.Color.BACKGROUND_BLACK: "bg:ansiblack",
             Token.Color.BACKGROUND_BLUE: "bg:ansiblue",
             Token.Color.BACKGROUND_CYAN: "bg:ansicyan",
@@ -324,7 +243,7 @@ DEFAULT_STYLE_DICT = LazyObject(
             Token.Color.INTENSE_RED: "ansibrightred",
             Token.Color.INTENSE_WHITE: "ansiwhite",
             Token.Color.INTENSE_YELLOW: "ansibrightyellow",
-            Token.Color.NO_COLOR: "noinherit",
+            Token.Color.RESET: "noinherit",
             Token.Color.PURPLE: "ansimagenta",
             Token.Color.RED: "ansired",
             Token.Color.UNDERLINE_BLACK: "underline ansiblack",
@@ -395,9 +314,6 @@ DEFAULT_STYLE_DICT = LazyObject(
             Token.Literal.String.Regex: "ansimagenta",
             Token.Literal.String.Single: "",
             Token.Literal.String.Symbol: "ansiyellow",
-            Token.Menu.Completions: "bg:ansigray ansiblack",
-            Token.Menu.Completions.Completion: "",
-            Token.Menu.Completions.Completion.Current: "bg:ansibrightblack ansiwhite",
             Token.Name: "",
             Token.Name.Attribute: "ansibrightyellow",
             Token.Name.Builtin: "ansigreen",
@@ -423,24 +339,18 @@ DEFAULT_STYLE_DICT = LazyObject(
             Token.Operator.Word: "bold ansimagenta",
             Token.Other: "",
             Token.Punctuation: "",
-            Token.Scrollbar: "bg:ansibrightblack",
-            Token.Scrollbar.Arrow: "bg:ansiblack ansiwhite bold",
-            Token.Scrollbar.Button: "bg:ansiblack",
             Token.Text: "",
             Token.Text.Whitespace: "ansigray",
+            Token.PTK.Aborting: "ansibrightblack",
+            Token.PTK.AutoSuggestion: "ansibrightblack",
+            Token.PTK.CompletionMenu: "bg:ansigray ansiblack",
+            Token.PTK.CompletionMenu.Completion: "",
+            Token.PTK.CompletionMenu.Completion.Current: "bg:ansibrightblack ansiwhite",
+            Token.PTK.Scrollbar.Arrow: "bg:ansiblack ansiwhite bold",
+            Token.PTK.Scrollbar.Background: "bg:ansibrightblack",
+            Token.PTK.Scrollbar.Button: "bg:ansiblack",
         },
     ),
     globals(),
     "DEFAULT_STYLE_DICT",
 )
-
-PTK2_STYLE = {
-    "completion-menu": "bg:ansigray ansiblack",
-    "completion-menu.completion": "",
-    "completion-menu.completion.current": "bg:ansibrightblack ansiwhite",
-    "scrollbar.background": "bg:ansibrightblack",
-    "scrollbar.arrow": "bg:ansiblack ansiwhite bold",
-    "scrollbar.button": "bg:ansiblack",
-    "auto-suggestion": "ansibrightblack",
-    "aborting": "ansibrightblack",
-}
